/*
* Copyright (c) Contributors, http://opensimulator.org/
* See CONTRIBUTORS.TXT for a full list of copyright holders.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the OpenSim Project nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
*/
using System;
using libsecondlife;

namespace OpenSim.Framework
{
    public class AgentCircuitData
    {
        public AgentCircuitData()
        {
        }

        public AgentCircuitData(sAgentCircuitData cAgent)
        {
            AgentID = new LLUUID(cAgent.AgentID);
            SessionID = new LLUUID(cAgent.SessionID);
            SecureSessionID = new LLUUID(cAgent.SecureSessionID);
            startpos = new LLVector3(cAgent.startposx, cAgent.startposy, cAgent.startposz);
            firstname = cAgent.firstname;
            lastname = cAgent.lastname;
            circuitcode = cAgent.circuitcode;
            child = cAgent.child;
            InventoryFolder = new LLUUID(cAgent.InventoryFolder);
            BaseFolder = new LLUUID(cAgent.BaseFolder);
            CapsPath = cAgent.CapsPath;
        }

        public LLUUID AgentID;
        public LLUUID SessionID;
        public LLUUID SecureSessionID;
        public LLVector3 startpos;
        public string firstname;
        public string lastname;
        public uint circuitcode;
        public bool child;
        public LLUUID InventoryFolder;
        public LLUUID BaseFolder;
        public string CapsPath = String.Empty;
    }

    [Serializable]
    public class sAgentCircuitData
    {
        public sAgentCircuitData()
        {
        }

        public sAgentCircuitData(AgentCircuitData cAgent)
        {
            AgentID = cAgent.AgentID.UUID;
            SessionID = cAgent.SessionID.UUID;
            SecureSessionID = cAgent.SecureSessionID.UUID;
            startposx = cAgent.startpos.X;
            startposy = cAgent.startpos.Y;
            startposz = cAgent.startpos.Z;
            firstname = cAgent.firstname;
            lastname = cAgent.lastname;
            circuitcode = cAgent.circuitcode;
            child = cAgent.child;
            InventoryFolder = cAgent.InventoryFolder.UUID;
            BaseFolder = cAgent.BaseFolder.UUID;
            CapsPath = cAgent.CapsPath;
        }

        public Guid AgentID;
        public Guid SessionID;
        public Guid SecureSessionID;
        public float startposx;
        public float startposy;
        public float startposz;
        public string firstname;
        public string lastname;
        public uint circuitcode;
        public bool child;
        public Guid InventoryFolder;
        public Guid BaseFolder;
        public string CapsPath = String.Empty;
    }
}